using System;
using Server;
using Server.Gumps;
using Server.Network;
using Server.Mobiles;

namespace Server.Gumps
{
	public enum ResurrectMessage
	{
		ChaosShrine = 0,
		VirtueShrine = 1,
		Healer = 2,
		Generic = 3,
	}

	public class ResurrectGump : Gump
	{
		private Mobile m_Healer;

		public ResurrectGump( Mobile owner ) : this( owner, owner, ResurrectMessage.Generic )
		{
		}

		public ResurrectGump( Mobile owner, Mobile healer ) : this( owner, healer, ResurrectMessage.Generic )
		{
		}

		public ResurrectGump( Mobile owner, ResurrectMessage msg ) : this( owner, owner, msg )
		{
		}

		public ResurrectGump( Mobile owner, Mobile healer, ResurrectMessage msg ) : base( 100, 0 )
		{
			m_Healer = healer;

			AddPage( 0 );

			AddBackground( 0, 0, 400, 350, 2600 );

			AddHtmlLocalized( 0, 20, 400, 35, 1011022, false, false ); // <center>Resurrection</center>

			AddHtmlLocalized( 50, 55, 300, 140, 1011023 + (int)msg, true, true ); /* It is possible for you to be resurrected here by this healer. Do you wish to try?<br>
																				   * CONTINUE - You chose to try to come back to life now.<br>
																				   * CANCEL - You prefer to remain a ghost for now.
																				   */

			AddButton( 200, 227, 4005, 4007, 0, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 235, 230, 110, 35, 1011012, false, false ); // CANCEL

			AddButton( 65, 227, 4005, 4007, 1, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 100, 230, 110, 35, 1011011, false, false ); // CONTINUE
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			Mobile from = state.Mobile;

			from.CloseGump( typeof( ResurrectGump ) );

			if ( info.ButtonID == 1 )
			{
				if ( from.Map == null || !from.Map.CanFit( from.Location, 16, false, false ) )
				{
					from.SendLocalizedMessage( 502391 ); // Thou can not be resurrected there!
					return;
				}

				from.PlaySound( 0x214 );
				from.FixedEffect( 0x376A, 10, 16 );

				from.Resurrect();

				if ( m_Healer != null && from != m_Healer )
				{
					VirtueLevel level = VirtueHelper.GetLevel( m_Healer, VirtueName.Compassion );

					switch ( level )
					{
						case VirtueLevel.Seeker: from.Hits = AOS.Scale( from.HitsMax, 20 ); break;
						case VirtueLevel.Follower: from.Hits = AOS.Scale( from.HitsMax, 40 ); break;
						case VirtueLevel.Knight: from.Hits = AOS.Scale( from.HitsMax, 80 ); break;
					}
				}

				Mobile m = from;

				Misc.Titles.AwardFame( from, -100, true ); // TODO: Proper fame loss

				if ( !Core.AOS && from.ShortTermMurders >= 5 )
				{
					double loss = (100.0 - ( 4.0 + (from.ShortTermMurders/5.0))) / 100.0;//5 to 15% loss
					if ( loss < 0.85 )
						loss = 0.85;
					else if ( loss > 0.95 )
						loss = 0.95;

					if ( from.RawStr * loss > 10 )
						from.RawStr = (int)(from.RawStr * loss);
					if ( from.RawInt * loss > 10 )
						from.RawInt = (int)(from.RawInt * loss);
					if ( from.RawDex * loss > 10 )
						from.RawDex = (int)(from.RawDex * loss);

					for (int s=0;s<from.Skills.Length;s++)
					{
						if ( from.Skills[s].Base * loss > 35 )
							from.Skills[s].Base *= loss;
					}
				}
			}
		}
	}
}